@@ -1,6 +1,6 @@
 /*
  *   libteam.c - Network team device driver library
- *   Copyright (C) 2011-2013 Jiri Pirko <jiri@resnulli.us>
+ *   Copyright (C) 2011-2015 Jiri Pirko <jiri@resnulli.us>
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -458,6 +458,9 @@ static int do_create(struct team_handle *th, const char *team_name, bool recreat
 		return -ENOMEM;
 
 	if (team_name) {
+		if (strlen(team_name) >= IFNAMSIZ)
+			return -ENAMETOOLONG;
+
 		rtnl_link_set_name(link, team_name);
 
 		if (recreate && team_ifname2ifindex(th, team_name)) {
