@@ -1,6 +1,6 @@
 /*
  *   teamd.c - Network team device daemon
- *   Copyright (C) 2011-2013 Jiri Pirko <jiri@resnulli.us>
+ *   Copyright (C) 2011-2015 Jiri Pirko <jiri@resnulli.us>
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -565,7 +565,7 @@ int teamd_loop_callback_timer_add_set(struct teamd_context *ctx,
 				      struct timespec *initial)
 {
 	int err;
-	int fd = fd;
+	int fd;
 
 	fd = timerfd_create(CLOCK_MONOTONIC, 0);
 	if (fd < 0) {
@@ -846,8 +846,14 @@ static int teamd_add_ports(struct teamd_context *ctx)
 
 	teamd_config_for_each_key(key, ctx, "$.ports") {
 		err = teamd_port_add_ifname(ctx, key);
-		if (err)
+		if (err == -ENODEV) {
+			teamd_log_warn("%s: Skipped adding a missing port.", key);
+			continue;
+		} else if (err) {
+			teamd_log_err("%s: Failed to add port (%s).", key,
+				      strerror(-err));
 			return err;
+		}
 	}
 	return 0;
 }
@@ -855,7 +861,7 @@ static int teamd_add_ports(struct teamd_context *ctx)
 static int teamd_hwaddr_check_change(struct teamd_context *ctx,
 				     struct teamd_port *tdport)
 {
-	char *hwaddr;
+	const char *hwaddr;
 	unsigned char hwaddr_len;
 	int err;
 
@@ -873,7 +879,6 @@ static int teamd_hwaddr_check_change(struct teamd_context *ctx,
 		teamd_log_err("Failed to set team device hardware address.");
 		return err;
 	}
-	ctx->hwaddr = hwaddr;
 	return 0;
 }
 
@@ -1442,23 +1447,23 @@ static int teamd_start(struct teamd_context *ctx, enum teamd_exit_code *p_ret)
 	pid_t pid;
 	int err = 0;
 
-	if (getuid() != 0) {
-		teamd_log_err("This program is intended to be run as root.");
-		return -EPERM;
-	}
+	if (getuid() == 0)
+		teamd_log_warn("This program is not intended to be run as root.");
 
 	if (daemon_reset_sigs(-1) < 0) {
 		teamd_log_err("Failed to reset all signal handlers.");
 		return -errno;
 	}
 
-	if (daemon_unblock_sigs(-1) < 0) {
+	if (daemon_unblock_sigs(SIGPIPE, -1) < 0) {
 		teamd_log_err("Failed to unblock all signals.");
 		return -errno;
 	}
 
 	pid = daemon_pid_file_is_running();
-	if (pid >= 0) {
+	if (pid == 0)
+		daemon_pid_file_remove();
+	if (pid > 0) {
 		teamd_log_err("Daemon already running on PID %u.", pid);
 		return -EEXIST;
 	}
@@ -1539,7 +1544,7 @@ static int teamd_generate_devname(struct teamd_context *ctx)
 {
 	char buf[IFNAMSIZ];
 	int i = 0;
-	uint32_t ifindex = ifindex;
+	uint32_t ifindex = 0; /* gcc needs this initialized */
 	int ret;
 	int err;
 
@@ -1614,10 +1619,6 @@ static int teamd_set_default_pid_file(struct teamd_context *ctx)
 	if (ctx->pid_file)
 		return 0;
 
-	err = teamd_make_rundir();
-	if (err)
-		return err;
-
 	err = asprintf(&ctx->pid_file, TEAMD_RUN_DIR"%s.pid", ctx->team_devname);
 	if (err == -1) {
 		teamd_log_err("Failed allocate memory for PID file string.");
@@ -1663,12 +1664,110 @@ static void teamd_context_fini(struct teamd_context *ctx)
 	free(ctx);
 }
 
+
+#ifdef HAVE_LIBCAP
+#include <sys/prctl.h>
+#include <sys/capability.h>
+#include <sys/types.h>
+#include <pwd.h>
+#include <grp.h>
+
+#ifndef TEAMD_USER
+#define TEAMD_USER "root"
+#endif
+#ifndef TEAMD_GROUP
+#define TEAMD_GROUP "root"
+#endif
+
+static int teamd_drop_privileges()
+{
+	cap_value_t cv[] = {CAP_NET_ADMIN, CAP_NET_BIND_SERVICE, CAP_NET_RAW};
+	cap_t my_caps;
+	struct passwd *pw = NULL;
+	struct group *grpent = NULL;
+
+	if ((pw = getpwnam(TEAMD_USER)) == NULL) {
+		fprintf(stderr, "Error reading user %s entry (%m)\n", TEAMD_USER);
+		goto error;
+	}
+
+	if (pw->pw_uid == 0)
+		return 0;
+
+	if ((grpent = getgrnam(TEAMD_GROUP)) == NULL) {
+		fprintf(stderr, "Error reading group %s entry (%m)\n", TEAMD_GROUP);
+		goto error;
+	}
+
+	if (pw->pw_gid != grpent->gr_gid) {
+		fprintf(stderr, "%s GID (%u) does not match %s GID (%u)\n",
+			TEAMD_USER, pw->pw_gid, TEAMD_GROUP, grpent->gr_gid);
+		goto error;
+	}
+
+	if (chown(TEAMD_RUN_DIR, pw->pw_uid, pw->pw_gid) < 0) {
+		fprintf(stderr, "Unable to change ownership of %s to %s/%s (%m)\n",
+			TEAMD_RUN_DIR, TEAMD_USER, TEAMD_GROUP);
+		goto error;
+	}
+
+	if (prctl(PR_SET_KEEPCAPS, 1, 0, 0, 0) < 0)
+		goto error;
+
+	if (setgid(pw->pw_gid) < 0) {
+		fprintf(stderr, "Unable to set process GID to %u (%m)\n", pw->pw_gid);
+		goto error;
+	}
+
+	if (initgroups(TEAMD_USER, pw->pw_gid) < 0) {
+		fprintf(stderr, "Unable to initialize the group access list for %s user with GID %u (%m)\n",
+			TEAMD_USER, pw->pw_gid);
+		goto error;
+	}
+	if (setuid(pw->pw_uid) < 0) {
+		fprintf(stderr, "Unable to set UID to %u (%m)\n", pw->pw_uid);
+		goto error;
+	}
+
+	if ((my_caps = cap_init()) == NULL)
+		goto error;
+	if (cap_set_flag(my_caps, CAP_EFFECTIVE, ARRAY_SIZE(cv), cv, CAP_SET) < 0)
+		goto error;
+	if (cap_set_flag(my_caps, CAP_PERMITTED, ARRAY_SIZE(cv), cv, CAP_SET) < 0)
+		goto error;
+	if (cap_set_proc(my_caps) < 0)
+		goto error;
+	cap_free(my_caps);
+
+	return 0;
+error:
+	fprintf(stderr, "Failed to drop privileges\n");
+	return -EINVAL;
+}
+
+#else
+
+static int teamd_drop_privileges()
+{
+	return 0;
+}
+
+#endif
+
 int main(int argc, char **argv)
 {
 	enum teamd_exit_code ret = TEAMD_EXIT_FAILURE;
 	int err;
 	struct teamd_context *ctx;
 
+	err = teamd_make_rundir();
+	if (err)
+		return ret;
+
+	err = teamd_drop_privileges();
+	if (err)
+		return ret;
+
 	err = teamd_context_init(&ctx);
 	if (err) {
 		fprintf(stderr, "Failed to init daemon context\n");
@@ -1729,15 +1828,20 @@ int main(int argc, char **argv)
 	case DAEMON_CMD_VERSION:
 		break;
 	case DAEMON_CMD_KILL:
-		err = daemon_pid_file_kill_wait(SIGTERM, 5);
-		if (err)
-			teamd_log_warn("Failed to kill daemon: %s", strerror(errno));
-		else
-			ret = TEAMD_EXIT_SUCCESS;
+		if (daemon_pid_file_is_running() > 0) {
+			err = daemon_pid_file_kill_wait(SIGTERM, 5);
+			if (err)
+				teamd_log_warn("Failed to kill daemon: %s",
+					       strerror(errno));
+			else
+				ret = TEAMD_EXIT_SUCCESS;
+		} else {
+			teamd_log_warn("Daemon not running");
+		}
 		break;
 	case DAEMON_CMD_CHECK:
-		ret = (daemon_pid_file_is_running() >= 0) ? TEAMD_EXIT_SUCCESS :
-							    TEAMD_EXIT_FAILURE;
+		ret = (daemon_pid_file_is_running() > 0) ? TEAMD_EXIT_SUCCESS :
+							   TEAMD_EXIT_FAILURE;
 		break;
 	case DAEMON_CMD_RUN:
 		err = teamd_start(ctx, &ret);
