@@ -1,6 +1,6 @@
 /*
  *   teamd_link_watch.c - Team port link watchers
- *   Copyright (C) 2012-2013 Jiri Pirko <jiri@resnulli.us>
+ *   Copyright (C) 2012-2015 Jiri Pirko <jiri@resnulli.us>
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 #include <sys/types.h>
 #include <netdb.h>
 #include <time.h>
+#include <limits.h>
 #include <private/misc.h>
 #include <team.h>
 
@@ -55,6 +56,7 @@ int __set_sockaddr(struct sockaddr *sa, socklen_t sa_len, sa_family_t family,
 	if (sa_len != result->ai_addrlen) {
 		/* This should not happen, so just to be safe */
 		teamd_log_err("Wrong address length in result.");
+		freeaddrinfo(result);
 		return -EINVAL;
 	}
 	memcpy(sa, result->ai_addr, sa_len);
@@ -89,6 +91,8 @@ int teamd_link_watch_check_link_up(struct teamd_context *ctx,
 	common_ppriv->link_up = new_link_up;
 	teamd_log_info("%s: %s-link went %s.", tdport->ifname, lw_name,
 		       new_link_up ? "up" : "down");
+	if (!new_link_up && common_ppriv->link_down_count < INT_MAX)
+		common_ppriv->link_down_count++;
 	return teamd_event_port_link_changed(ctx, tdport);
 }
 
@@ -180,6 +184,16 @@ static int link_watch_state_up_get(struct teamd_context *ctx,
 	return 0;
 }
 
+static int link_watch_state_down_count_get(struct teamd_context *ctx,
+					   struct team_state_gsc *gsc,
+					   void *priv)
+{
+	struct lw_common_port_priv *common_ppriv = priv;
+
+	gsc->data.int_val = common_ppriv->link_down_count;
+	return 0;
+}
+
 static const struct teamd_state_val link_watch_state_vals[] = {
 	{
 		.subpath = "name",
@@ -191,6 +205,11 @@ static const struct teamd_state_val link_watch_state_vals[] = {
 		.type = TEAMD_STATE_ITEM_TYPE_BOOL,
 		.getter = link_watch_state_up_get,
 	},
+	{
+		.subpath = "down_count",
+		.type = TEAMD_STATE_ITEM_TYPE_INT,
+		.getter = link_watch_state_down_count_get,
+	},
 };
 
 static const struct teamd_state_val link_watch_state_vg = {
