@@ -30,6 +30,7 @@
 #include <errno.h>
 #include <team.h>
 #include <private/misc.h>
+#include <net/ethernet.h>
 
 #include "teamd.h"
 #include "teamd_config.h"
@@ -60,6 +61,7 @@ struct lacpdu_info {
 } __attribute__((__packed__));
 
 struct lacpdu {
+	struct ether_header	hdr;
 	uint8_t			subtype;
 	uint8_t			version_number;
 	uint8_t			actor_tlv_type;
@@ -295,7 +297,7 @@ static int lacp_load_config(struct teamd_context *ctx, struct lacp *lacp)
 	return 0;
 }
 
-static bool lacp_port_selectable(struct lacp_port *lacp_port)
+static bool lacp_port_loopback_free(struct lacp_port *lacp_port)
 {
 	if (!memcmp(lacp_port->actor.system,
 		    lacp_port->partner.system, ETH_ALEN)) {
@@ -303,11 +305,24 @@ static bool lacp_port_selectable(struct lacp_port *lacp_port)
 			       "team device.", lacp_port->tdport->ifname);
 		return false;
 	}
+	return true;
+}
+
+static bool lacp_port_selectable_state(struct lacp_port *lacp_port)
+{
 	if (lacp_port->state == PORT_STATE_CURRENT)
 		return true;
 	return false;
 }
 
+static bool lacp_port_unselectable_state(struct lacp_port *lacp_port)
+{
+	if (lacp_port->state == PORT_STATE_CURRENT ||
+	    lacp_port->state == PORT_STATE_EXPIRED)
+		return false;
+	return true;
+}
+
 static bool lacp_port_selected(struct lacp_port *lacp_port)
 {
 	return lacp_port->agg_lead;
@@ -724,13 +739,15 @@ static bool lacp_port_mergeable(struct lacp_port *lacp_port)
 static int lacp_port_agg_update(struct lacp_port *lacp_port)
 {
 	if (lacp_port_selected(lacp_port) &&
-	    (!lacp_port_selectable(lacp_port) ||
+	    (lacp_port_unselectable_state(lacp_port) ||
+	     !lacp_port_loopback_free(lacp_port) ||
 	     !lacp_port_correct_aggregation(lacp_port) ||
 	     lacp_port_mergeable(lacp_port)))
 		lacp_port_agg_unselect(lacp_port);
 
 	if (!lacp_port_selected(lacp_port) &&
-	    lacp_port_selectable(lacp_port))
+	    (lacp_port_selectable_state(lacp_port) &&
+	     lacp_port_loopback_free(lacp_port)))
 		lacp_port_agg_select(lacp_port);
 
 	return lacp_selected_agg_update(lacp_port->lacp, NULL);
@@ -903,7 +920,7 @@ static void lacp_port_actor_update(struct lacp_port *lacp_port)
 		state |= INFO_STATE_EXPIRED;
 	if (lacp_port->state == PORT_STATE_DEFAULTED)
 		state |= INFO_STATE_DEFAULTED;
-	if (teamd_port_count(lacp_port->ctx) > 1)
+	if (teamd_port_count(lacp_port->ctx) > 0)
 		state |= INFO_STATE_AGGREGATION;
 	teamd_log_dbg("%s: lacp info state: 0x%02X.", lacp_port->tdport->ifname,
 						      state);
@@ -1013,6 +1030,8 @@ static int lacpdu_send(struct lacp_port *lacp_port)
 	struct lacpdu lacpdu;
 	struct sockaddr_ll ll_my;
 	struct sockaddr_ll ll_slow;
+	char *hwaddr;
+	unsigned char hwaddr_len;
 	int err;
 	bool admin_state;
 
@@ -1028,12 +1047,19 @@ static int lacpdu_send(struct lacp_port *lacp_port)
 
 	memcpy(lacp_port->actor.system, lacp_port->ctx->hwaddr, ETH_ALEN);
 
+	hwaddr = team_get_ifinfo_orig_hwaddr(lacp_port->tdport->team_ifinfo);
+	hwaddr_len = team_get_ifinfo_orig_hwaddr_len(lacp_port->tdport->team_ifinfo);
+	if (hwaddr_len != ETH_ALEN)
+		return 0;
+
 	lacpdu_init(&lacpdu);
 	lacpdu.actor = lacp_port->actor;
 	lacpdu.partner = lacp_port->partner;
+	memcpy(lacpdu.hdr.ether_shost, hwaddr, hwaddr_len);
+	memcpy(lacpdu.hdr.ether_dhost, ll_slow.sll_addr, ll_slow.sll_halen);
+	lacpdu.hdr.ether_type = htons(ETH_P_SLOW);
 
-	err = teamd_sendto(lacp_port->sock, &lacpdu, sizeof(lacpdu), 0,
-			   (struct sockaddr *) &ll_slow, sizeof(ll_slow));
+	err = teamd_send(lacp_port->sock, &lacpdu, sizeof(lacpdu), 0);
 	return err;
 }
 
@@ -1201,9 +1227,9 @@ static int lacp_port_added(struct teamd_context *ctx,
 		return err;
 	}
 
-	err = teamd_packet_sock_open(&lacp_port->sock,
-				     tdport->ifindex,
-				     htons(ETH_P_SLOW), NULL, NULL);
+	err = teamd_packet_sock_open_type(SOCK_RAW, &lacp_port->sock,
+					  tdport->ifindex,
+					  htons(ETH_P_SLOW), NULL, NULL);
 	if (err)
 		return err;
 
